/*
 * Copyright (C) 2007 - AlannY
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/** 
* @file digest.c
* @brief Digest
* @author AlannY
* @date 2007-10-11
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

#include <dbug.h>

#include "digest.h"

inline void digest_free(char *data) {
    DBUG_ENTER("digest_free");

    assert(data);

    if( data )
        free(data);

    DBUG_VOID_RETURN;
}

inline void mdX_encode(uint8_t *output, uint32_t *input, uint32_t len) {
    uint32_t i, j;

    assert(output);
    assert(input);

    for( i = 0, j = 0; j < len; i++, j += 4) {
        output[j] = (uint8_t)(input[i] & 0xff);
        output[j+1] = (uint8_t)((input[i] >> 8) & 0xff);
        output[j+2] = (uint8_t)((input[i] >> 16) & 0xff);
        output[j+3] = (uint8_t)((input[i] >> 24) & 0xff);
    }
}

inline void mdX_decode(uint32_t *output, uint8_t *input, uint32_t len) {
    uint32_t i, j;

    assert(output);
    assert(input);

    for( i = 0, j = 0; j < len; i++, j += 4 )
        output[i] = ((uint32_t)input[j]) | (((uint32_t)input[j+1]) << 8) | (((uint32_t)input[j+2]) << 16) | (((uint32_t)input[j+3]) << 24);
}

