/*
 * Copyright (C) 2007 - AlannY
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/** 
* @file queue.c
* @brief Queue
* @author AlannY
* @date 2007-10-15
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <assert.h>

#include <dbug.h>

#include <gtk/gtk.h>
#include <pthread.h>

#include "readhash.h"
#include "entry.h"

#include "ff/crc32.h"
#include "ff/md4.h"
#include "ff/md5.h"
#include "ff/sha1.h"
#include "ff/sha256.h"
#include "ff/sha512.h"
#include "ff/ed2k.h"
#include "ff/tiger.h"

#include "gui/window.h"

#include "queue.h"

queue_t *queue;
entry_t *cur_entry;

pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

inline void queue_start() {
    pthread_t thread;
    pthread_create(&thread, NULL, queue_init, NULL);
}

void* queue_init(void *ptr) {
    
    queue = malloc(sizeof(queue_t));
    memset(queue, 0, sizeof(queue_t));

    while( 1 ) {
        if( status->flags & STATUS_QUIT ) {
            break;
        }

        entry_t *entry = queue_get(queue);

        if( entry == NULL ) {
            continue;
        }
        
        cur_entry = entry;
        status->flags &= ~STATUS_ABORT;

        switch( entry->what ) {
            case 1:
                if( entry->crc32 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*9);
                    memset(digest, 0, sizeof(uint8_t)*9);
                    if( crc32_digest(entry->path, digest) ) {
                        entry->crc32 = digest;
                        check_crc_in_table(entry);
                    } else {
                        goto abort;
                    }
                }
                break;
            case 2:
                if( entry->md4 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*33);
                    memset(digest, 0, sizeof(uint8_t)*33);
                    if( md4_digest(entry->path, digest) ) {
                        entry->md4 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 3:
                if( entry->md5 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*33);
                    memset(digest, 0, sizeof(uint8_t)*33);
                    if( md5_digest(entry->path, digest) ) {
                        entry->md5 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 4:
                if( entry->sha1 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*41);
                    memset(digest, 0, sizeof(uint8_t)*41);
                    if( sha1_digest(entry->path, digest) ) {
                        entry->sha1 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 5:
                if( entry->sha256 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*65);
                    memset(digest, 0, sizeof(uint8_t)*65);
                    if( sha256_digest(entry->path, digest) ) {
                        entry->sha256 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
             case 6:
                if( entry->sha384 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*97);
                    memset(digest, 0, sizeof(uint8_t)*97);
                    if( sha384_digest(entry->path, digest) ) {
                        entry->sha384 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 7:
                if( entry->sha512 == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*129);
                    memset(digest, 0, sizeof(uint8_t)*129);
                    if( sha512_digest(entry->path, digest) ) {
                        entry->sha512 = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 8:
                if( entry->ed2k == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*33);
                    memset(digest, 0, sizeof(uint8_t)*33);
                    if( ed2k_digest(entry->path, digest) ) {
                        entry->ed2k = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            case 9:
                if( entry->ed2klink == NULL ) {
                    if( entry->ed2k == NULL ) {
                        uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*33);
                        memset(digest, 0, sizeof(uint8_t)*33);
                        if( ed2k_digest(entry->path, digest) ) {
                            entry->ed2k = digest;
                        } else {
                            goto abort;
                        }
                    }
                    entry->ed2klink = ed2k_link(entry->path, entry->ed2k);
                }
                break;
            case 10:
                if( entry->tiger == NULL ) {
                    uint8_t *digest = (uint8_t*)malloc(sizeof(uint8_t)*49);
                    memset(digest, 0, sizeof(uint8_t)*49);
                    if( tiger_digest(entry->path, digest) ) {
                        entry->tiger = digest;
                    } else {
                        goto abort;
                    }
                }
                break;
            default:
                break;
        }
        entry->what = 0;
        show_text_wrap(entry);

        abort:
        ;
    }
    void* rc = NULL;
    return rc;
}

inline void queue_flush(queue_t *q) {
    DBUG_ASSERT(q);

    entry_queue_t *entry, *entry1;
  
    pthread_mutex_lock( &mutex );

    for(entry = q->first_entry; entry != NULL; entry = entry1) {
        entry1 = entry->next;
        free(entry);
    }
    
    q->last_entry = NULL;
    q->first_entry = NULL;
    q->count = 0;

    pthread_mutex_unlock( &mutex );
}

inline void queue_remove(queue_t *q, entry_t *entry) {
    DBUG_ASSERT(q);
    DBUG_ASSERT(entry);

    pthread_mutex_lock( &mutex );
    
    entry_queue_t *entry_queue = q->first_entry;
    entry_queue_t *prev = q->first_entry;

    while( entry_queue != NULL ) {
        if( entry_queue->entry == entry ) {
            if( entry_queue == q->first_entry ) {
                q->first_entry = entry_queue->next;
            } else {
                prev->next = entry_queue->next;
            }

            if( !q->first_entry ) {
                q->last_entry = NULL;
            }

            q->count--;
            free(entry_queue);
            break;
        }
        prev = entry_queue;
        entry_queue = entry_queue->next;
    }

    pthread_mutex_unlock( &mutex );
}

inline void queue_put(queue_t *q, entry_t *entry) {
    DBUG_ASSERT(q);
    DBUG_ASSERT(entry);

    entry_queue_t *entry_queue = malloc(sizeof(entry_queue_t));
    memset(entry_queue, 0, sizeof(entry_queue_t));

    entry_queue->entry = entry;
    entry_queue->next = NULL;
 
    pthread_mutex_lock( &mutex );

    if( !q->last_entry )
        q->first_entry = entry_queue;
    else
        q->last_entry->next = entry_queue;

    q->last_entry = entry_queue;
    q->count++;

    pthread_cond_signal( &cond );
    pthread_mutex_unlock( &mutex );
}

inline entry_t* queue_get(queue_t *q) {
    DBUG_ASSERT(q);

    pthread_mutex_lock( &mutex );
    
    entry_queue_t *entry_queue = NULL;
    entry_t *entry =  NULL;

    for(;;) {
        if( status->flags & STATUS_QUIT ) {
            break;
        }

        entry_queue = q->first_entry;
        
        if( entry_queue != NULL ) {
            q->first_entry = entry_queue->next;
            if( !q->first_entry ) {
                q->last_entry = NULL;
            }

            q->count--;
            
            entry = entry_queue->entry;
            
            free(entry_queue);

            break;
        } else {
            pthread_cond_wait( &cond, &mutex );
        }
    }

    pthread_mutex_unlock( &mutex );
    
    return entry;
}

