/*
 * Copyright (C) 2007 - AlannY
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * aint32_t32_t64_t with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/** 
* @file readhash.c
* @brief readhash
* @author AlannY
* @date 2007-10-09
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <stdlib.h>
#include <string.h>

#include <dbug.h>

#include "i18n.h"

#ifdef USE_GTK
#include "gtk/gtk.h"
#endif

#ifdef USE_GNOMEVFS
#include <libgnomevfs/gnome-vfs.h>
#endif

#include "entry.h"
#include "queue.h"
#include "gui/window.h"
#include "cli.h"

#include "readhash.h"

status_t *status;

status_t* default_status() {
    DBUG_ENTER("default_status");

    status_t *status = (status_t*)malloc(sizeof(status_t));
    memset(status, 0, sizeof(status_t));

    status->path = NULL;

    status->flags &= ~STATUS_AUTO_CRC32;
    status->flags &= ~STATUS_QUIT;
    status->flags &= ~STATUS_GUI;

    DBUG_RETURN(status);
}

inline void status_free(status_t *status) {
    DBUG_ENTER("status_free");

    if( status->path )
        free(status->path);
    if( status )
        free(status);

    DBUG_VOID_RETURN;
}

int main(int argc, char *argv[]) {
    DBUG_PUSH("d:t:O");
    DBUG_PROCESS("readhash");
    
    DBUG_ENTER("main");

    status = default_status();

    bindtextdomain(PROGRAM, LOCALEDIR);
    textdomain(PROGRAM);

#ifdef USE_GNOMEVFS
    gnome_vfs_init();
#endif

    if( cli(&argc, &argv) == 1 ) {
        goto quit;
    }

    queue_start();

#ifdef USE_GTK
    init_gui(&argc, &argv);
#endif

    quit:
    status_free(status);
    
    DBUG_RETURN(EXIT_SUCCESS);
}

